@@ -27,4 +27,4 @@ mod types;
 #[doc(inline)]
 pub use key::{DerivedKey, EncryptedDerivedKey, EncryptedRootKey, Key, Nonce, RootKey};
 #[doc(inline)]
-pub use types::{CipherText, ClearText, ConstArray};
+pub use types::{CipherText, ClearText};
