@@ -4,7 +4,6 @@
 //!
 //! This module deals in low level cryptographic details. It is advisable to not deal with this module
 //! directly, and instead use a higher level API.
-
 use argon2::Argon2;
 use blake3::Hasher;
 use chacha20::{
@@ -12,12 +11,13 @@ use chacha20::{
     XChaCha20, XNonce,
 };
 use rand::{Rng, RngCore};
+use redacted::RedactedBytes;
 use serde::{Deserialize, Serialize};
 use snafu::{ensure, ResultExt};
 use zeroize::{Zeroize, Zeroizing};
 
 use crate::{
-    crypto::types::{CipherText, ClearText, ConstArray},
+    crypto::types::{CipherText, ClearText},
     error::{Argon2Failure, BackendError, BadHMAC},
 };
 
@@ -36,7 +36,7 @@ pub trait Key {
 /// Nonces are intended to be always randomly generated, and there is intentionally no API for
 /// reconstructing a nonce.
 #[derive(Debug, Hash, Clone, PartialEq, Eq, PartialOrd, Ord, Serialize, Deserialize)]
-pub struct Nonce(pub(crate) ConstArray<24>);
+pub struct Nonce(pub(crate) RedactedBytes<24>);
 
 impl Nonce {
     /// Generates a new, random nonce
@@ -69,11 +69,11 @@ impl Nonce {
 #[zeroize(drop)]
 pub struct RootKey {
     /// The root cipher key, used for encrypting headers
-    encryption: ConstArray<32>,
+    encryption: RedactedBytes<32>,
     /// The root HMAC key, used for validating headers
-    hmac: ConstArray<32>,
+    hmac: RedactedBytes<32>,
     /// Random data used to derive encryption keys
-    entropy: ConstArray<256>,
+    entropy: RedactedBytes<256>,
 }
 
 impl RootKey {
@@ -275,9 +275,9 @@ pub struct EncryptedRootKey {
     /// The nonce used for the encryption
     nonce: Nonce,
     /// The HMAC tag
-    hmac: ConstArray<32>,
+    hmac: RedactedBytes<32>,
     /// The salt used for argon2
-    salt: ConstArray<32>,
+    salt: RedactedBytes<32>,
     /// The encrypted payload
     #[serde(with = "serde_bytes")]
     payload: Vec<u8>,
@@ -334,9 +334,9 @@ impl EncryptedRootKey {
 #[zeroize(drop)]
 pub struct DerivedKey {
     /// The encryption key
-    encryption: ConstArray<32>,
+    encryption: RedactedBytes<32>,
     /// The root HMAC key, used for validating headers
-    hmac: ConstArray<32>,
+    hmac: RedactedBytes<32>,
     /// The context string used to produce this key
     context_string: String,
 }
