@@ -5,16 +5,13 @@
 //! This module deals in low level cryptographic details. It is advisable to not deal with this module
 //! directly, and instead use a higher level API.
 
-use std::{
-    borrow::Cow,
-    io::Cursor,
-    ops::{Deref, DerefMut},
-};
+use std::{borrow::Cow, io::Cursor};
 
 use chacha20::{
     cipher::{NewCipher, StreamCipher},
     XChaCha20,
 };
+use redacted::RedactedBytes;
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 use snafu::{ensure, ResultExt};
 use zeroize::Zeroize;
@@ -128,7 +125,7 @@ pub struct CipherText<'a> {
     /// The nonce used for the encryption
     pub(crate) nonce: Nonce,
     /// The HMAC tag
-    pub(crate) hmac: ConstArray<32>,
+    pub(crate) hmac: RedactedBytes<32>,
     /// The encrypted payload
     #[serde(serialize_with = "serde_bytes::serialize")]
     pub(crate) payload: Cow<'a, [u8]>,
@@ -172,72 +169,6 @@ impl CipherText<'_> {
     }
 }
 
-/// Thin wrapper around a const-generic array to make them work better with serde
-#[derive(Debug, Hash, Clone, PartialEq, Eq, PartialOrd, Ord, Zeroize)]
-#[zeroize(drop)]
-pub struct ConstArray<const N: usize>(pub [u8; N]);
-
-impl<const N: usize> Deref for ConstArray<N> {
-    type Target = [u8];
-
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
-impl<const N: usize> DerefMut for ConstArray<N> {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.0
-    }
-}
-
-impl<const N: usize> From<[u8; N]> for ConstArray<N> {
-    fn from(x: [u8; N]) -> Self {
-        Self(x)
-    }
-}
-
-impl<const N: usize> From<ConstArray<N>> for [u8; N] {
-    fn from(val: ConstArray<N>) -> Self {
-        val.0
-    }
-}
-
-impl<const N: usize> Serialize for ConstArray<N> {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: serde::Serializer,
-    {
-        let bytes = serde_bytes::Bytes::new(&self.0);
-        bytes.serialize(serializer)
-    }
-}
-
-impl<const N: usize> AsRef<[u8; N]> for ConstArray<N> {
-    fn as_ref(&self) -> &[u8; N] {
-        &self.0
-    }
-}
-
-impl<'de, const N: usize> Deserialize<'de> for ConstArray<N> {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: serde::Deserializer<'de>,
-    {
-        let bytes = serde_bytes::ByteBuf::deserialize(deserializer)?;
-        let mut new_bytes = [0_u8; N];
-        if bytes.len() == N {
-            new_bytes.copy_from_slice(&bytes);
-            Ok(Self(new_bytes))
-        } else {
-            Err(serde::de::Error::invalid_length(
-                bytes.len(),
-                &"Wrong length",
-            ))
-        }
-    }
-}
-
 /// Unit tests
 #[cfg(test)]
 mod tests {
