@@ -1,37 +1,4 @@
-//! Encrypted document storage backed for the
-//! [`snapper`](https://gitlab.com/rust-community-matrix/snapper) ecosystem.
-//!
-//! This crate implements a multi-file, namespaced, log structured merge tree, with encrypted entries,
-//! where each namespace has its own file.
-//!
-//! This crate encrypts LSM entries with `XChaCha20`, and then uses Blake3 as an HMAC (an
-//! Encrypt-then-MAC construction), with no configuration in the cipher or hash, and uses a different
-//! derived (via Blake3) key for each name space.
-//!
-//! The root key is generated randomly, and then encrypted with an argon2 derivation of the provided
-//! password. This crate does not provide support for plaintext operation, passwordless operation should
-//! be achieved by using a null password.
-//!
-//! This crate provides optional support for zstd compressed entries.
-//!
-//! Serialized entries are stored as [CBOR](https://cbor.io/).
-//!
-//! # FAQ
-//!
-//! ## Why doesn't this use poly1305?
-//!
-//! The author of this crate has a deep distrust of polynomial MACs, in particular, but by no means
-//! limited to, the [non-committing nature](https://www.youtube.com/watch?v=h-T1bQTt4_Y&t=735s) of
-//! AEAD's based on a polynomial MAC.
-//!
-//! The implemented encryption scheme (XChaCha20 + Blake3 Encrypt-then-MAC) is commiting, unlike the
-//! non-commiting XChaCha20 + Poly1305.
-//!
-//! ## What does <span style="color:red">**HAZMAT**</span> mean?
-//!
-//! This tag indicates that the module that bears it contains hazardous materials, namely, the low level
-//! cryptography underpinnings of this module. These are dangerously easy to misuse, and you probably
-//! don't want to interact with them directly.
+#![doc = include_str!("../README.md")]
 #![warn(
     clippy::all,
     clippy::pedantic,
@@ -59,121 +26,30 @@ use std::{
 };
 
 use serde::{de::DeserializeOwned, Serialize};
-use snafu::{ensure, OptionExt, ResultExt, Snafu};
+use snafu::{ensure, OptionExt, ResultExt};
 use tracing::{info, instrument};
 
 use crate::{
     crypto::{DerivedKey, EncryptedRootKey, RootKey},
     entries::{Control, Namespace, Namespaces, Settings},
+    error::{
+        CryptoBoxError, DirectoryAlreadyExists, DirectoryDoesNotExist, FailedCreatingDirectory,
+        Fetch, MissingConfiguration, MissingNamespaceDirectory, NamespaceOpen, RootKeyDecryption,
+        RootKeyEncryption, RootKeyIO, RootKeySerial, RootNamespaceInit, RootNamespaceOpen, Store,
+    },
     file::LsmFile,
 };
 
+#[cfg(feature = "experimental-async")]
+pub mod async_wrapper;
 pub mod crypto;
-pub mod entries;
+mod entries;
 pub mod error;
 pub mod file;
 
-/// Errors that can be returned when interacting with a [`CryptoBox`]
-#[derive(Debug, Snafu)]
-#[non_exhaustive]
-pub enum CryptoBoxError {
-    /// Directory already exists
-    #[snafu(display("Directory already exists: {}", directory))]
-    DirectoryAlreadyExists {
-        /// The directory that already exists
-        directory: String,
-    },
-    /// Failed creating the `CryptoBox` directory
-    FailedCreatingDirectory {
-        /// The directory we couldn't create
-        directory: String,
-        /// The underlying IO error
-        source: std::io::Error,
-    },
-    /// Failed to encrypt root key
-    RootKeyEncryption {
-        /// The underlying error
-        source: error::Error,
-    },
-    /// Failed to decrypt root key. Most likely this is an incorrect password.
-    RootKeyDecryption {
-        /// The underlying error
-        source: error::Error,
-    },
-    /// Failed to write root key file
-    #[snafu(display("Failed to write to path: {}", path))]
-    RootKeyIO {
-        /// The path we failed to write to
-        path: String,
-        /// The underlying error
-        source: std::io::Error,
-    },
-    /// [`RootKey`] serialization error
-    RootKeySerial {
-        /// The underlying error
-        source: serde_cbor::Error,
-    },
-    /// Failed to initialize the root namespace
-    #[snafu(display("Failed to initialize root namespace: {}", path))]
-    RootNamespaceInit {
-        /// The path of the root namespace
-        path: String,
-        /// The underlying error
-        source: crate::error::Error,
-    },
-    /// Failed to open root namespace
-    #[snafu(display("Failed to open root namespace: {}", path))]
-    RootNamespaceOpen {
-        /// The path of the root namespace
-        path: String,
-        /// The underlying error
-        source: crate::error::Error,
-    },
-    /// Path does not exist or is not a directory
-    #[snafu(display("Path does not exist or is not a directory: {}", path))]
-    DirectoryDoesNotExist {
-        /// The path that doesn't exist
-        path: String,
-    },
-    /// Missing namespace directory
-    MissingNamespaceDirectory,
-    /// `CryptoBox` is missing the configuration entry
-    MissingConfiguration,
-    /// Failed to open a namespace
-    #[snafu(display("Failed to open a namespace {}", name))]
-    NamespaceOpen {
-        /// The name of the namespace that failed to open
-        name: String,
-        /// The underlying error
-        source: crate::error::Error,
-    },
-    /// No such namespace
-    #[snafu(display("No such namespace: {}", name))]
-    NoSuchNamespace {
-        /// Namespace that doesn't exist
-        name: String,
-    },
-    /// Error retrieving item
-    Fetch {
-        /// Underlying failure
-        source: crate::error::Error,
-    },
-    /// Error storing item
-    Store {
-        /// Underlying failure
-        source: crate::error::Error,
-    },
-    /// An error occurred while flushing the `CryptoBox`
-    #[snafu(display("One or more errors occurred flushing: {:?}", sources))]
-    Flush {
-        /// Underlying failures
-        sources: Vec<(Option<String>, crate::error::Error)>,
-    },
-}
-
 /// The `CryptoBox` encrypted, namespaced document store
 ///
-/// This will default to
+/// See module level documentation for more information.
 pub struct CryptoBox {
     /// The path for this CryptoBox
     path: PathBuf,
@@ -482,6 +358,25 @@ impl CryptoBox {
             })
         }
     }
+
+    /// Gets the specified item from the root namespace, if it exists
+    ///
+    /// # Arguments
+    ///
+    ///   * `key` - Key for the key/value pair to be retrieved
+    ///
+    /// # Errors
+    ///
+    ///   * [`CryptoBoxError::Fetch`] if an underlying error occurs
+    #[instrument(skip(self, key), err)]
+    pub fn get_root<K, V>(&mut self, key: &K) -> Result<Option<V>, CryptoBoxError>
+    where
+        K: Serialize,
+        V: DeserializeOwned,
+    {
+        self.root_namespace.get(key).context(Fetch)
+    }
+
     /// Stores the specified item from the specified namespace
     ///
     /// # Arguments
@@ -515,6 +410,25 @@ impl CryptoBox {
         }
     }
 
+    /// Stores the specified item in the root namespace
+    ///
+    /// # Arguments
+    ///
+    ///   * `key` - Key for the key/value pair to be stored
+    ///   * `value` - Value for the key/value pair to be stored
+    ///
+    /// # Errors
+    ///
+    ///   * [`CryptoBoxError::Store`] if an underlying error occurs
+    #[instrument(skip(self, key, value), err)]
+    pub fn insert_root<K, V>(&mut self, key: &K, value: &V) -> Result<(), CryptoBoxError>
+    where
+        K: Serialize,
+        V: Serialize,
+    {
+        self.root_namespace.insert(key, value).context(Store)
+    }
+
     /// Returns true if the specified namespace contains the provided key
     ///
     /// # Arguments
@@ -582,6 +496,40 @@ impl CryptoBox {
             })
         }
     }
+
+    /// Gets all the key/value pairs in a namespace as [`Vec`] of pairs
+    ///
+    /// # Errors
+    ///
+    /// Will return [`CryptoBoxError::Fetch`] if there are any underlying errors.
+    pub fn to_pairs<K, V>(&mut self, namespace: &str) -> Result<Vec<(K, V)>, CryptoBoxError>
+    where
+        K: DeserializeOwned + Serialize + Eq + Clone,
+        V: DeserializeOwned + Clone,
+    {
+        if let Some((_, lsm)) = self.namespaces.get_mut(namespace) {
+            // Store the key in the lsmfile
+            lsm.to_pairs().context(Fetch)
+        } else {
+            Err(CryptoBoxError::NoSuchNamespace {
+                name: namespace.to_string(),
+            })
+        }
+    }
+
+    /// Gets all the key/value pairs in the root namespace as a [`Vec`] of pairs
+    ///
+    /// # Errors
+    ///
+    /// Will return [`CryptoBoxError::Fetch`] if there are any underlying errors.
+    pub fn root_to_pairs<K, V>(&mut self) -> Result<Vec<(K, V)>, CryptoBoxError>
+    where
+        K: DeserializeOwned + Serialize + Eq + Clone,
+        V: DeserializeOwned + Clone,
+    {
+        // Store the key in the lsmfile
+        self.root_namespace.to_pairs().context(Fetch)
+    }
 }
 
 /// Unit tests for `CryptoBox`
@@ -738,6 +686,34 @@ mod tests {
             );
             Ok(())
         }
+        /// Test basic insertions with `pairs` export
+        #[test]
+        fn basic_insertions_pairs() -> Result<(), CryptoBoxError> {
+            let tempdir = tempdir().context(FailedCreatingDirectory {
+                directory: "tempdir".to_string(),
+            })?;
+            let path = tempdir.path().join("box");
+            let pairs = [(1, 2), (3, 4), (5, 6)];
+            // Initialize the box
+            let mut crypto_box = CryptoBox::init(&path, None, None, "testing")?;
+            // Add the empty string namespace
+            crypto_box.create_namespace("".to_string())?;
+            // do the insertions
+            let namespace = "";
+            for (key, value) in &pairs {
+                crypto_box.insert(key, value, namespace)?;
+            }
+            let comparison = pairs.into_iter().collect::<HashMap<i32, i32>>();
+            assert_eq!(crypto_box.to_hashmap("")?, comparison,);
+            assert_eq!(
+                crypto_box
+                    .to_pairs::<i32, i32>("")?
+                    .into_iter()
+                    .collect::<HashMap<_, _>>(),
+                comparison
+            );
+            Ok(())
+        }
     }
     /// Test failure modes of the box
     mod failures {
